#include "ldap.h"
#include "popen.h"
#include "daemoni.h"

/*

Small routines which are not LDAP spesific.

*/

/*
	Return value of keyed item, removing the key from the config line
*/

const char *get_keyval (VIEWITEMS &items, const char *key )
{
	const char *ret = NULL;
	VIEWITEM *it = items.locate (key);
	if (it != NULL){
		const char *pt = it->line.get();
		pt = str_skip(pt);
		pt += strlen(key);
		pt = str_skip (pt);
		ret =pt;		
	}
	return ret;
}

/*
	Set key and value in the config line
	If value is empty, remove key and value, save comment.
*/

void set_keyval (VIEWITEMS &items, const char *key, const char *val)
{
	VIEWITEM *it = items.locate (key);		
	if (it == NULL){
		it = new VIEWITEM(key);
		items.add (it);
	}
		if (strcmp(val,"")==0) {
		it->line.setfrom ("");
	}
	else {
		char line[1000];
		snprintf (line,sizeof(line)-1,"%s\t%s",key,val);
		it->line.setfrom (line);
	}
}

/* OS functions */

/*
	Run raw system command
*/
int sys_command_raw (
	const char *cmd,
	const char *args,
	SSTRINGS &tb)
{
	int ret = -1;
	POPEN pop (cmd,args);
	if (pop.isok()){
		char buf[1000];
		buf[0] = buf[1] = buf[2] = ' ';
		while (pop.wait(10) > 0){
			while (pop.readout(buf+3,sizeof(buf)-4)!=-1){
				tb.add (new SSTRING(buf));
			}
		}
		ret = 0;
	}else{
		xconf_error (MSG_U(E_CANTEXEC,"Can't execute the command\n%s"),cmd);
	}
	return ret;
}

/*
	Run system command
*/

int sys_command (
	const char *cmd,
	const char *args,
	SSTRINGS &tb)
{
	int ret = -1;
	char title[PATH_MAX+100];
	DAEMON_INTERNAL *dae = daemon_find (cmd);
	if (dae != NULL){
		snprintf (title,sizeof(title)-1
			,"%s: %s %s",MSG_U(I_OUTPUT,"Output of command")
			,dae->getpath(),args);
		tb.add (new SSTRING(title));
		ret = sys_command_raw (cmd,args,tb);
	}
	return ret;
}

/*
	View a system file
*/

int sys_showfile (
	const char *file,
	SSTRINGS &tb)
{
	char title[PATH_MAX+100];
	snprintf (title,sizeof(title)-1,"%s %s",MSG_U(I_FILE,"File"),file);
	tb.add (new SSTRING(title));
	return sys_command_raw ("cat",file,tb);
}

/*
	Run system command, and show result dialog
*/

void sys_command_title (
	const char *title,
	const char *cmd,
	const char *args)
{
	SSTRINGS tb;
	if (sys_command (cmd,args,tb) != -1){
		dialog_textbox (title,tb);
	}
}
