/*********** 

LDAPCONF

Licence: Linuxconf/GPL (see the file COPYING for details)
Author: Stein Vrle <stein@terminator.net>

************/

#include "ldap.h"

/*
	LDAPDB
	This object holds properties for a LDAP database.
*/
PUBLIC LDAPDB::LDAPDB ()
{

}

PUBLIC LDAPDB::LDAPDB (const char *dbname)
{
  	name = dbname; 
}

/*
	LDAPDB - export
*/
PUBLIC int LDAPDB::export_ldif()
{
	int ret=0;
	char dbfile[PATH_MAX] = "";
	sprintf (dbfile,"%s/id2entry.dbb",directory.get());
	char exportfile [PATH_MAX] = "";
	sprintf (exportfile,"/tmp/%s.ldif",name);
	SSTRING filename(exportfile);
	int nof;
	DIALOG dia;
	dia.newf_str (MSG_U(F_EXPORT_LDIF_FILE,"LDIF Filename"),exportfile,40);
	while (1){
		MENU_STATUS code = dia.edit (MSG_U(T_EXPORT_LDIF,"LDIF Export")
			,MSG_U(I_EXPORT_LDIF
				,"This will export all entries from this database\ninto a LDIF file.")
				,help_ldap			
				,nof);
		if (code == MENU_CANCEL || code == MENU_ESCAPE){
			break;
		}
		else if (code == MENU_ACCEPT){
			char arg[1000] = "";
			sprintf (arg,"-n %s > %s",dbfile,exportfile);
			sys_command_title("LDIF Export","ldbmcat",arg);
			break;
		}
	}	
	return ret;
}

/*
	LDAPDB - import_ldif

	This is designed to import a Netscape exported addressbook in LDIF format.
	Currently it just add the suffix to the database and import.
*/
PUBLIC int LDAPDB::import_ldif()
{
	int ret=0;
	char add_suffix=true;
	char add_suffixobject=true;

	char conffile[PATH_MAX] = "";
	sprintf (conffile,"%s/%s",LDAP_CONF_DIR,name);
	char importfile [PATH_MAX] = "";
	sprintf (importfile,"/tmp/%s.ldif",name);
	SSTRING filename(importfile);
	int nof;
	DIALOG dia;
	dia.newf_str (MSG_U(F_IMPORT_LDIF_FILE,"LDIF Filename"),importfile,40);
	dia.newf_chk ("Add domainsuffix to DN attributes",add_suffix,"DN Suffix");
	dia.newf_chk ("Add domainobject to database",add_suffixobject,"DB Suffix");
	while (1){
		MENU_STATUS code = dia.edit (MSG_U(T_IMPORT_LDIF,"LDIF Import")
			,MSG_U(I_IMPORT_LDIF
				,"This will import entries into this database\nfrom a LDIF file.")
				,help_ldap			
				,nof);
		if (code == MENU_CANCEL || code == MENU_ESCAPE){
			break;
		}
		else if (code == MENU_ACCEPT){
			char arg[1000] = "";
			
			/* Prepare directory: Add a domain object, and also add this to every imported DN */ 
			char tmpfile[PATH_MAX] = "/tmp/import.ldif.tmp";	
			FILE *fin = fopen(importfile,"r");
			FILE *fout = fopen (tmpfile,"w");
		        if ((fin != NULL)&&(fout != NULL)) {
		                char buf[1000];
				SSTRING line;
				if (add_suffixobject==true) { 
				  fprintf (fout,"dn: %s\n",bind.base.get());
				  fprintf (fout,"dc: %s\n",bind.base.get());
				  fprintf (fout,"objectclass: top\n");
				  fprintf (fout,"objectclass: domain\n");
				}
				fprintf (fout,"\n");
				while (fgets(buf,sizeof(buf)-1,fin)!=NULL){
					strip_end(buf);
		                        line.setfrom(buf);
					// xconf_notice(buf);
					if ((line.strstr("dn:")!=NULL)&&(add_suffixobject==true)) { /* Add suffix to DN */
						char newdn[1000];
						sprintf(newdn,",%s",bind.base.get());	
						line.append(newdn);
					}
                                	fprintf (fout,"%s\n",line.get());
                            	}
				fclose (fout);
				fclose (fin);
           			/* Import into database */
				sprintf (arg,"-i %s -f %s",tmpfile,conffile);
				sys_command_title("LDIF Import","ldif2ldbm",arg);
			} else {}
			break;
			
		}
	}	
	return ret;
}

/* 
	Database config
*/

PUBLIC void LDAPDB::config(const char *fname)
{
VIEWITEMS v_db_conf;
VIEWITEMS v_slapd_conf;

char was_enabled=false;
char enable_database=false;        
char path[PATH_MAX] = "";
sprintf (path,"%s/%s",LDAP_CONF_DIR,fname);
CONFIG_FILE f_db_conf (path
		,help_ldap
                ,CONFIGF_OPTIONNAL|CONFIGF_MANAGED
		,"root","root",0400
                ,subsys_ldap);
	
	name = fname;


/* Read config */
        v_db_conf.read (f_db_conf);
	bind.base.setfrom (get_keyval(v_db_conf,"suffix"));
	bind.dn.setfrom (get_keyval(v_db_conf,"rootdn"));
	bind.pw.setfrom (get_keyval(v_db_conf,"rootpw"));
	database.setfrom (get_keyval(v_db_conf,"database"));
	directory.setfrom (get_keyval(v_db_conf,"directory"));
	replogfile.setfrom (get_keyval(v_db_conf,"replogfile"));
	updatedn.setfrom (get_keyval(v_db_conf,"updatedn"));
	replica.setfrom (get_keyval(v_db_conf,"replica"));	
	lastmod.setfrom (get_keyval(v_db_conf,"lastmod"));
	readonly.setfrom (get_keyval(v_db_conf,"readonly"));

/* Check if this db is enabled in slapd.conf */
	VIEWITEMS inc_items;
	VIEWITEM *inc_item=NULL;
	v_slapd_conf.read (f_slapd_conf);
	int n = v_slapd_conf.locate("include",inc_items); 
	for (int i=0; i<n; i++){
	  inc_item = inc_items.getitem(i);
	    if (inc_item->line.strstr(fname)!=0) {
		enable_database = true;
		was_enabled = true;
		break; /* inc_item will point to this db entry */
	  }
        }

/* Draw dialog */
	SSTRING filename(fname);
	DIALOG dia;
	dia.newf_chk (MSG_U(F_DB_ENABLE,"Enable database"),enable_database,"");
	dia.newf_str (MSG_U(F_DB_PROFILE,"Filename"),filename);
	dia.newf_title ("",MSG_U(I_MASTER_DATABASE,"Database settings"));
	FIELD_COMBO *dblist = dia.newf_combo(MSG_U(F_SLAPD_DATABASE,"Database"),database);
		dblist->addopt ("ldbm");
		dblist->addopt ("shell");
		dblist->addopt ("passwd");
	dia.newf_str (MSG_R(F_API_BASE),bind.base);
	dia.newf_str (MSG_R(F_API_DN),bind.dn);
	dia.newf_str (MSG_R(F_API_PW),bind.pw);
	dia.newf_str (MSG_U(F_DB_DIRECTORY,"Data directory"),directory);	
	dia.newf_str (MSG_U(F_DB_REPLOG,"Replicaton log"),replogfile);
	dia.newf_str (MSG_U(F_DB_UPDATEDN,"Update DN"),updatedn);	
	dia.newf_str (MSG_U(F_DB_REPLICA,"Replica host"),replica);	
	FIELD_COMBO *lastmod_key= dia.newf_combo (MSG_U(F_DB_LASTMOD,"Lastmod recording"),lastmod);
		lastmod_key->addopt ("on");
		lastmod_key->addopt ("off");
	FIELD_COMBO *readonly_key = dia.newf_combo (MSG_U(F_DB_READONLY,"Readonly"),readonly);
		readonly_key->addopt ("on");
		readonly_key->addopt ("off");

	dia.setbutinfo (MENU_USR1,MSG_U(B_EXPORT,"Export")
			,MSG_U(X_EXPORT,"Export"));
	dia.setbutinfo (MENU_USR2,MSG_U(B_IMPORT,"Import")
			,MSG_U(X_IMPORT,"Import"));
/* Dialog */
	int nof = 0;
	while (1){
		MENU_STATUS code = dia.edit (MSG_U(T_DATABASE_MENU,"LDAP database")
			,MSG_U(I_DATABASE_MENU
				,"This is the settings for this LDAP database.\nCurrently ldapconf will save database settings in individual files,\n and include the files in the main slapd.conf file.\nYou may still have database settings in the main file\n, but ldapconf will not operate on this for now.")				
			,help_ldap
			,nof
			,MENUBUT_ACCEPT|MENUBUT_CANCEL);

/* Exit */	if (code == MENU_CANCEL || code == MENU_ESCAPE){
			break;
		}
/* Export */	else if (code == MENU_USR1){	
			export_ldif();
		}
/* Import */	else if (code == MENU_USR2){
			import_ldif();
		}
/* Save */	else if (code == MENU_ACCEPT){
			set_keyval(v_db_conf,"suffix",bind.base.get());
			set_keyval(v_db_conf,"rootdn",bind.dn.get());
			set_keyval(v_db_conf,"rootpw",bind.pw.get());
			set_keyval(v_db_conf,"database",database.get());
			set_keyval(v_db_conf,"directory",directory.get());
			set_keyval(v_db_conf,"replogfile",replogfile.get());
			set_keyval(v_db_conf,"updatedn",updatedn.get());
			set_keyval(v_db_conf,"replica",replica.get());
			set_keyval(v_db_conf,"lastmod",lastmod.get());
			set_keyval(v_db_conf,"readonly",readonly.get());
			char path[PATH_MAX] = "";
			fname = filename.get();
			sprintf (path,"%s/%s",LDAP_CONF_DIR,fname);
			CONFIG_FILE f_db_conf_save (path
				,help_ldap
                		,CONFIGF_OPTIONNAL|CONFIGF_MANAGED
				,"root","root",0400
                		,subsys_ldap);
			v_db_conf.write (f_db_conf_save,&p_ldap);
			if (enable_database && !was_enabled) {				
				char buf[1000];
				VIEWITEM *it;
				sprintf(buf,"%s %s","include",path);
				it = new VIEWITEM(buf);
				v_slapd_conf.add(it); /* add db entry */
			}
			else if (was_enabled && !enable_database) {
			  inc_item->line.setfrom(""); /* remove db entry */
			}

			v_slapd_conf.write(f_slapd_conf,&p_ldap);
			break;
		}
	}
}

/* 
	Database command menu 
*/
PUBLIC void LDAPDB::menu()
{
	const char *config_key = MSG_U(M_DB_CONFIG,"Configure database");
	const char *export_key = MSG_U(M_DB_EXPORT,"Export database to LDIF file");
	const char *import_key = MSG_U(M_DB_IMPORT,"Import LDIF file into database");
	const char *create_key = MSG_U(M_DB_CREATE,"Migrate system data into database");

	DIALOG_MENU dia;
	dia.new_menuinfo("Database name",name);
	dia.new_menuitem("Config",config_key);
	dia.new_menuitem("Export LDIF",export_key);
	dia.new_menuitem("Import LDIF",import_key);
	dia.new_menuitem("Import system",create_key);
	int nof = 0;
	while (1){
		MENU_STATUS code = dia.editmenu (MSG_U(T_DB_MENU,"Database menu")
			,MSG_U(I_DB_MENU,"Select operation to perform on this database")
			,help_ldap
			,nof,0);
		if (code == MENU_ESCAPE || code == MENU_QUIT){
			break;
		}else{
			const char *key = dia.getmenustr(nof);
			if (key == config_key){
				config(name);
			}else if (key == export_key){
				export_ldif();
			}else if (key == import_key){
				import_ldif();
			}else if (key == create_key){
				ldap_system_migrate();
			}
		}
	}
}

